package no.skarasoftwaredevelopment.schooltracker.handlers;

import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.pojo.Achievement;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import android.content.Context;

public class AchievementHandler {

	public static final String TAG = "AchievementHandler";
	public static final int CHEST = 1;
	public static final int ATTENDANCE = 2;
	public static final int COIN = 3;
	public static final int TAP = 4;
	
	private static final AchievementType[] CHESTS;
	private static final AchievementType[] ATTENDANCES;
	private static final AchievementType[] COINS;
	private static final AchievementType[] TAPS;

	static {
		CHESTS = new AchievementType[4];
		CHESTS[0] = AchievementType.CHEST_1;
		CHESTS[1] = AchievementType.CHEST_10;
		CHESTS[2] = AchievementType.CHEST_25;
		CHESTS[3] = AchievementType.CHEST_50;
		
		ATTENDANCES = new AchievementType[4];
		ATTENDANCES[0] = AchievementType.ATTENDANCE_1;
		ATTENDANCES[1] = AchievementType.ATTENDANCE_10;
		ATTENDANCES[2] = AchievementType.ATTENDANCE_25;
		ATTENDANCES[3] = AchievementType.ATTENDANCE_50;
		
		COINS = new AchievementType[4];
		COINS[0] = AchievementType.COIN_100;
		COINS[1] = AchievementType.COIN_250;
		COINS[2] = AchievementType.COIN_500;
		COINS[3] = AchievementType.COIN_1000;
		
		TAPS = new AchievementType[4];
		TAPS[0] = AchievementType.TAP_10;
		TAPS[1] = AchievementType.TAP_50;
		TAPS[2] = AchievementType.TAP_100;
		TAPS[3] = AchievementType.TAP_150;
	}
	
	// defines the different achievement we have
	// should probably be defined in the database
	// or completely defined here
	public enum AchievementType {
		NONE(0),
		
		// chests
		CHEST_1(CHEST, 1, 1),
		CHEST_10(CHEST, 2, 10),
		CHEST_25(CHEST, 3, 25),
		CHEST_50(CHEST, 4, 50),
		
		// attendances
		ATTENDANCE_1(ATTENDANCE, 5, 1),
		ATTENDANCE_10(ATTENDANCE, 6, 10),
		ATTENDANCE_25(ATTENDANCE, 7, 25),
		ATTENDANCE_50(ATTENDANCE, 8, 50),
		
		// coins
		COIN_100(COIN, 9, 100),
		COIN_250(COIN, 10, 250),
		COIN_500(COIN, 11, 500),
		COIN_1000(COIN, 12, 1000),
		
		// taps
		TAP_10(TAP, 13, 10),
		TAP_50(TAP, 14, 50),
		TAP_100(TAP, 15, 100),
		TAP_150(TAP, 16, 150);
		
		private int category;
		private int id;
		private int amount;

		private AchievementType(int category, int id, int amount) {
			this.category = category;
			this.id = id;
			this.amount = amount;
		}
		
		private AchievementType(int id) {
			this.id = id;
		}
		
		public int getCategory() {
			return category;
		}
		
		public int getId() {
			return id;
		}
		
		public int getAmount() {
			return amount;
		}

	}
	
	/**
	 * Returns the correct {@code AchivementType} given the {@code Achievement}.
	 * 
	 * @param a
	 * @return
	 */
	public static AchievementType getAchievementTypeFromAchievement(Achievement a) {
		for (AchievementType at : AchievementType.values()) {
			if (a.getId() == at.getId()) {
				return at;
			}
		}
		return AchievementType.NONE;
	}
	
	/**
	 * Returns the correct AchivementType given the achievement id.
	 * 
	 * @param a
	 * @return
	 */
	public static AchievementType getAchievementTypeFromId(int achievementId) {
		for (AchievementType at : AchievementType.values()) {
			if (achievementId == at.getId()) {
				return at;
			}
		}
		return AchievementType.NONE;
	}
	
	/**
	 * Checks what achievements the student already has and returns a list of
	 * new achievements if there is any.
	 * 
	 * @param c
	 * @param studentId
	 * @return
	 */
	public static List<Achievement> checkAchievements(Context c, int studentId) {
		DbHelper db = new DbHelper(c);
		
		// gets the student's current achievements
		List<Achievement> currentAchievements = db.getStudentAchievements(studentId);
		
		// creates a list of achievements to check for
		List<AchievementType> checkAchievements = new ArrayList<AchievementType>();
		for (AchievementType at : AchievementType.values()) {
			// we check if the current list of achievements contains this type
			boolean containsAchievement = false;
			for (Achievement a : currentAchievements) {
				if (a.getId() == at.getId()) {
					containsAchievement = true;
					break;
				}
			}
			
			// we add the achievement type to the list if it doesn't
			if (!containsAchievement) {
				checkAchievements.add(at);
			}
		}
		
		List<AchievementType> newAchievements = new ArrayList<AchievementType>();

		int numberOfChests = db.getNumberOfChests(studentId);
		int numberOfAttendances = db.getTotalAttendancesAttended(studentId);
		
		Student student = db.getStudent(1); // TODO: get studentId dynamically
		int numberOfCoins = student.getCoinTotal();
		int numberOfTaps = student.getTaps(); 
		
		// loop through the AchievementTypes the student don't yet have
		// and add the applicable ones to the list
		for (AchievementType newAchievement : checkAchievements) {
			switch (newAchievement.getCategory()) {
				case CHEST:
					if (check(CHESTS, newAchievement, numberOfChests)) newAchievements.add(newAchievement);
					break;
				case ATTENDANCE:
					if (check(ATTENDANCES, newAchievement, numberOfAttendances)) newAchievements.add(newAchievement);
					break;
				case COIN:
					if (check(COINS, newAchievement, numberOfCoins)) newAchievements.add(newAchievement);
					break;
				case TAP:
					if (check(TAPS, newAchievement, numberOfTaps)) newAchievements.add(newAchievement);
					break;
				default:
					break;
			}
		}
		
		// gets a list of Achievements instead of AchievementTypes from the database
		return db.getAchievements(newAchievements);
	}
	
	private static boolean check(AchievementType[] types, AchievementType newAchievement, int amount) {		
		for (AchievementType type : types) {
			if (newAchievement.getId() == type.getId() && amount >= type.getAmount()) {
				return true;
			}
		}
		return false;
	}

	public static void checkEligibility(DbHelper db, Context c) {
		List<Achievement> achievements = checkAchievements(c, 1); // TODO: get studentId dynamically 
		
		if (achievements.size() > 0) {
			db.addStudentAchievements(1, achievements); // TODO: dynamically find studentId
			NotificationHandler.notifyAchievement(c, achievements.size());
		}
	}
	
}