package no.skarasoftwaredevelopment.schooltracker.parsers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import no.skarasoftwaredevelopment.schooltracker.pojo.Coord;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData;
import android.util.JsonReader;

/**
 * This class will parse the URL in MAZEMAP and retrieve the
 * correct position URL to use according to where (which campus)
 * the student is currently located.
 * 
 * For Glshaugen, the position URL provided by the site resolver
 * would be: https://ntnu2-pos.mazemap.com/ntnu/position
 * 
 * However, if the user isn't connected to the wireless network
 * at NTNU it will cause the web server to throw an 404 resulting
 * in an IOException. The same exception will get thrown if the
 * user is connected to the wireless network but MazeMap reports
 * an error (caused by Cisco) instead of the actual position.
 * 
 * The last possibility for an error to get thrown by this class
 * is if the student is not connected to the Internet at all.
 * 
 * @author Tommy
 *
 */
public final class PositionParser {

	public static final String TAG = "PositionParser";
	
	private static final String MAZEMAP = "http://use.mazemap.com/siteresolver/"; // MazeMap siteresolver URL
	//private static final String MAZEMAP = "http://hv-6141.idi.ntnu.no/schooltracker/siteresolver.json";
    
	/**
	 * This method either returns the parsed data from MazeMap
	 * or a null indicating that we got an error message or we
	 * weren't able to connect to MazeMap.
	 * 
	 * @return
	 */
    public static PositionData parsePosition() {
		String positionUrl = null;
		JsonReader reader = null;
		
		try {			
			URL url = new URL(MAZEMAP);
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginObject();
			if (reader.hasNext()) {
				String name = reader.nextName();
				if (name.equals("positionUrl")) {
					positionUrl = reader.nextString();
				}
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {}
			}
		}

		return parsePositionData(positionUrl);
    }
	
	private static PositionData parsePositionData(String positionUrl) {
		PositionData data = null;
		
		if (positionUrl != null) {
			JsonReader reader = null;
			
			String changedOn = null;
			double confidenceFactor = 0.0;
			String elem = null;
			double geoLatitude = Coord.DUMMY_GEO_LAT;
			double geoLongitude = Coord.DUMMY_GEO_LON;
			double x = 0.0;
			double y = 0.0;
			double z = 0.0;
			String logTimestamp = null;
			String error = null;  
			
			try {			
				URL url = new URL(positionUrl);
				BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
				reader = new JsonReader(in);
				
				reader.beginObject();
				while (reader.hasNext()) {
					String name = reader.nextName();
					if (name.equals("changedOn")) {
						changedOn = reader.nextString();
					} else if (name.equals("confidencefactor")) {
						confidenceFactor = reader.nextDouble();
					} else if (name.equals("elem")) {
						elem = reader.nextString();
					} else if (name.equals("geoLatitude")) {
						geoLatitude = reader.nextDouble();
					} else if (name.equals("geoLongitude")) {
						geoLongitude = reader.nextDouble();
					} else if (name.equals("x")) {
						x = reader.nextDouble();
					} else if (name.equals("y")) {
						y = reader.nextDouble();
					} else if (name.equals("z")) {
						z = reader.nextDouble();
					} else if (name.equals("logtimestamp")) {
						logTimestamp = reader.nextString();
					} else if (name.equals("error")) {
						error = reader.nextString();
					} else {
						reader.skipValue();
					}
				}
				reader.endObject();
				
				// unless we got an error from MazeMap, we create a PositionData object
				if (logTimestamp == null && error == null) {
					data = new PositionData(changedOn,
							confidenceFactor, elem, geoLongitude, geoLatitude,
							x, y, z);
				}
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {}
				}
			}
		}

		return data;
	}
	
}