package no.skarasoftwaredevelopment.schooltracker.parsers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.pojo.Coord;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import android.util.JsonReader;
import android.util.JsonToken;

public final class RoomParser {

	public static final String TAG = "RoomParser";
	public static final String ROOMS = "http://hv-6141.idi.ntnu.no/schooltracker/app_get_all_rooms.php";
	public static final String ROOM = "http://hv-6141.idi.ntnu.no/schooltracker/room_";
	//public static final String NTNU_ROOMS = "http://www.ime.ntnu.no/api/fdv/rooms"; // NTNU IME API
	//public static final String NTNU_ROOM = "http://www.ime.ntnu.no/api/fdv/rooms/lydiacode:"; // NTNU IME API

	/**
	 * Parses the manually edited room database on the webserver for rooms
	 * that contains GPS coordinates representing a boundary around a given
	 * room, and also includes what floor the room is on.
	 * 
	 * @return
	 */
	public static List<Room> parseRooms() {
		URL url = null;
		BufferedReader in = null;
		JsonReader reader = null;
		List<Room> rooms = null;
		
		try {
			url = new URL(ROOMS);
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginArray();
			rooms = readRoomArray(reader);			
			reader.endArray();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {}
			}
		}
	
		return rooms;
	}

	private static List<Room> readRoomArray(JsonReader reader) throws IOException {
		List<Room> rooms = new ArrayList<Room>();
		
		while (reader.hasNext()) {
			rooms.add(readRoom(reader));
		}
		
		return rooms;
	}

	private static Room readRoom(JsonReader reader) throws IOException {
		int id = -1;
		String nr = null;
		String name = null;
		String type = null;
		int floorId = -1;
		int buildingId = -1;
		String lydiaCode = null;
		Coord geoBottomLeft = null;
		Coord geoTopLeft = null;
		Coord geoTopRight = null;
		Coord geoBottomRight = null;
		float floor = 0;
		
		reader.beginObject();
		while (reader.hasNext()) {
			String nextName = reader.nextName();
			if (nextName.equals("id")) {
				id = reader.nextInt();
			} else if (nextName.equals("nr")) {
				nr = reader.nextString();
			} else if (nextName.equals("name")) {
				name = reader.nextString();
			} else if (nextName.equals("type")) {
				type = reader.nextString();
			} else if (nextName.equals("building_id")) {
				floorId = reader.nextInt();
			} else if (nextName.equals("floor_id")) {
				buildingId = reader.nextInt();
			} else if (nextName.equals("lydia_code")) {
				lydiaCode = reader.nextString();
			} else if (nextName.equals("geo_bottom_left")) {
				geoBottomLeft = readCoord(reader);
			} else if (nextName.equals("geo_top_left")) {
				geoTopLeft = readCoord(reader);
			} else if (nextName.equals("geo_top_right")) {
				geoTopRight = readCoord(reader);
			} else if (nextName.equals("geo_bottom_right")) {
				geoBottomRight = readCoord(reader);
			} else if (nextName.equals("floor")) {
				floor = (float) reader.nextDouble();
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();
		
		Coord[] geoBoundary = new Coord[4];
		geoBoundary[0] = geoBottomLeft;
		geoBoundary[1] = geoTopLeft;
		geoBoundary[2] = geoTopRight;
		geoBoundary[3] = geoBottomRight;

		return new Room(id, nr, name, type, floorId, buildingId, lydiaCode, geoBoundary, floor);
	}
	
	/**
	 * Method for translating the spatial datatype Point in MySQL
	 * to the class Coord used by the system.
	 * 
	 * @param reader
	 * @return
	 * @throws IOException
	 */
	private static Coord readCoord(JsonReader reader) throws IOException {
		// format is: POINT(lon, lat)
		String rawCoords = reader.nextString();
		
		rawCoords = rawCoords.substring(6).replace(")", ""); // skip "POINT(" and remove ")"
		String[] rawCoordsList = rawCoords.split(" "); // gets the longitude and latitude
		
		return new Coord(Double.parseDouble(rawCoordsList[0]), Double.parseDouble(rawCoordsList[1]));
	}
	
	/**
	 * NOT IN USE.
	 * 
	 * Parses a single room given the lydia code.
	 * Does not contain boundary data or floor.
	 * 
	 * @param lydiaCode
	 * @return
	 */
	public static Room parseNTNURoom(String lydiaCode) {
		URL url = null;
		BufferedReader in = null;
		JsonReader reader = null;
		Room room = null;
		
		try {
			//url = new URL(ROOM + lydiaCode); // TODO: use this if NTNU IME API is selected
			url = new URL(ROOM + lydiaCode + ".json");
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginObject();
			reader.nextName(); // rooms
			
			reader.beginArray();
			room = readNTNURoomWithLydia(reader, lydiaCode);
			reader.endArray();
			
			// let's skip the "cache" section in the JSON data
			
			reader.nextName(); // cache
			reader.beginObject();
			while (reader.hasNext()) {
				reader.skipValue();
			}
			reader.endObject();

			reader.endObject();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (reader != null) {
					reader.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	
		return room;
	}

	/**
	 * NOT IN USE.
	 * 
	 * @param reader
	 * @param lydiaCode
	 * @return
	 * @throws IOException
	 */
	private static Room readNTNURoomWithLydia(JsonReader reader, String lydiaCode) throws IOException {
		int id = -1;
		String nr = null;
		String name = null;
		String type = null;
		int floorId = -1;
		int buildingId = -1;

		reader.beginObject();
		while (reader.hasNext()) {
			String nextName = reader.nextName();
			if (nextName.equals("id")) {
				id = reader.nextInt();
			} else if (nextName.equals("nr")) {
				nr = reader.nextString();
			} else if (nextName.equals("name")) {
				if (reader.peek() != JsonToken.NULL) {
					name = reader.nextString();
				} else {
					reader.nextNull();
				}
			} else if (nextName.equals("type")) {
				type = reader.nextString();
			} else if (nextName.equals("buildingId")) {
				floorId = reader.nextInt();
			} else if (nextName.equals("floorId")) {
				buildingId = reader.nextInt();
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new Room(id, nr, name, type, floorId, buildingId, lydiaCode);
	}
	
	/**
	 * NOT IN USE.
	 * 
	 * This methods takes all the rooms from the IME
	 * database at NTNU and parses them.
	 * 
	 * Does not include rooms with lydiaCode (null instead).
	 * 
	 * @return
	 */
	public static List<Room> parseNTNURooms() {
		URL url = null;
		BufferedReader in = null;
		JsonReader reader = null;
		List<Room> rooms = null;
		
		try {
			url = new URL(ROOMS);
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginObject();
			reader.nextName(); // rooms
			
			rooms = readNTNURoomArray(reader);
			
			// let's skip the "cache" section in the JSON data
			
			reader.nextName(); // cache
			reader.beginObject();
			while (reader.hasNext()) {
				reader.skipValue();
			}
			reader.endObject();

			reader.endObject();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	
		return rooms;
	}

	/**
	 * NOT IN USE.
	 * 
	 * @param reader
	 * @return
	 * @throws IOException
	 */
	private static List<Room> readNTNURoomArray(JsonReader reader) throws IOException {
		List<Room> rooms = new ArrayList<Room>();

		reader.beginArray();
		while (reader.hasNext()) {
			rooms.add(readNTNURoom(reader, null));
		}
		reader.endArray();
		
		return rooms;
	}

	/**
	 * NOT IN USE.
	 * 
	 * @param reader
	 * @param lydiaCode
	 * @return
	 * @throws IOException
	 */
	private static Room readNTNURoom(JsonReader reader, String lydiaCode) throws IOException {
		int id = -1;
		String nr = null;
		String name = null;
		String type = null;
		int floorId = -1;
		int buildingId = -1;

		reader.beginObject();
		while (reader.hasNext()) {
			String nextName = reader.nextName();
			if (nextName.equals("id")) {
				id = reader.nextInt();
			} else if (nextName.equals("nr")) {
				nr = reader.nextString();
			} else if (nextName.equals("name")) {
				if (reader.peek() != JsonToken.NULL) {
					name = reader.nextString();
				} else {
					reader.nextNull();
				}
			} else if (nextName.equals("type")) {
				type = reader.nextString();
			} else if (nextName.equals("buildingId")) {
				floorId = reader.nextInt();
			} else if (nextName.equals("floorId")) {
				buildingId = reader.nextInt();
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new Room(id, nr, name, type, floorId, buildingId, lydiaCode);
	}

}