package no.skarasoftwaredevelopment.schooltracker.parsers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.concurrent.TimeUnit;

import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.CourseActivity;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.Schedule;
import no.skarasoftwaredevelopment.schooltracker.pojo.Timetable;
import no.skarasoftwaredevelopment.schooltracker.pojo.activities.ActivityType;
import android.util.JsonReader;
import android.util.JsonToken;

public final class ScheduleParser {

	public static final String TAG = "ScheduleParser";
	public static final String SCHEDULE = "http://hv-6141.idi.ntnu.no/schooltracker/schedule_";
	//public static final String SCHEDULE = "http://www.ime.ntnu.no/api/schedule/"; // NTNU IME API
	
	public static Schedule parseSchedule(String courseCode) {
		URL url = null;
		BufferedReader in = null;
		JsonReader reader = null;
		List<CourseActivity> activities = null;
		
		try {
			url = new URL(SCHEDULE + courseCode + ".json");
			//url = new URL(SCHEDULE + courseCode); // NTNU
			
			in = new BufferedReader(new InputStreamReader(url.openStream()));
			reader = new JsonReader(in);
			
			reader.beginObject();
			reader.nextName(); // activity
			
			activities = readSchedule(reader);
			
			// let's skip the "cache" and "request" sections in the JSON data
			
			if (reader.peek() != JsonToken.NULL) {
				reader.nextName(); // cache
				reader.beginObject();
				while (reader.hasNext()) {
					reader.skipValue();
				}
				reader.endObject();
			}

			if (reader.peek() != JsonToken.NULL) {
				reader.nextName(); // request
				reader.beginObject();
				while (reader.hasNext()) {
					reader.skipValue();
				}
				reader.endObject();
			}

			if (reader.peek() != JsonToken.NULL) {
				reader.endObject();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {}
			}
		}
	
		return new Schedule(courseCode, activities);
	}

	private static List<CourseActivity> readSchedule(JsonReader reader) throws IOException {
		List<CourseActivity> activities = new ArrayList<CourseActivity>();

		JsonToken test = reader.peek();
		if (test != JsonToken.NULL) {
			reader.beginArray();
			while (reader.hasNext()) {
				activities.add(readActivityArray(reader));
			}
			reader.endArray();
		}
		
		return activities;
	}

	private static CourseActivity readActivityArray(JsonReader reader) throws IOException {
		String activityId = null;
		ActivityType type = null; // activity description
		String courseVersionCode = null;
		List<String> studyPrograms = new ArrayList<String>();
		List<Timetable> timetables = new ArrayList<Timetable>();

		reader.beginObject();
		while (reader.hasNext()) {
			String name = reader.nextName();
			if (name.equals("activityId")) {
				activityId = reader.nextString();
			} else if (name.equals("courseVersionCode")) {
				courseVersionCode = reader.nextString();
			} else if (name.equals("activityDescription")) {
				type = ActivityType.getType(reader.nextString());
			} else if (name.equals("studyProgrammes") || name.equals("studyPrograms")) {
				// there's a "typo" (or British English) used in the NTNU IME API, and the rest of
				// the world tend to use "program" instead, which is why we support both just in case
				studyPrograms = readStudyProgramsArray(reader);
			} else if (name.equals("activitySchedules") && reader.peek() != JsonToken.NULL) {
				timetables = readActivitySchedulesArray(reader);
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new CourseActivity(activityId, courseVersionCode, type, studyPrograms, timetables);
	}
	
	private static List<String> readStudyProgramsArray(JsonReader reader) throws IOException {
		List<String> studyPrograms = new ArrayList<String>();
        
		reader.beginArray();
		while (reader.hasNext()) {
			studyPrograms.add(reader.nextString());
		}
		reader.endArray();
		
		return studyPrograms;
	}
	
	private static List<Timetable> readActivitySchedulesArray(JsonReader reader) throws IOException {
		List<Timetable> timetables = new ArrayList<Timetable>();

		reader.beginArray();
		while (reader.hasNext()) {
			timetables.add(readActivitySchedule(reader));
		}
		reader.endArray();
		
		return timetables;
	}
	
	private static Timetable readActivitySchedule(JsonReader reader) throws IOException {
		int dayNumber = -1;
		String start = null;
		String end = null;
		List<Room> room = null;
		List<Attendance> weeks = null;
		
		reader.beginObject();
		while (reader.hasNext()) {
			String name = reader.nextName();
			if (name.equals("dayNumber")) {
				dayNumber = reader.nextInt();
			} else if (name.equals("start")) {
				start = reader.nextString().trim(); // removes white space
			} else if (name.equals("end")) {
				end = reader.nextString().trim(); // removes white space
			} else if (name.equals("rooms")) {
				room = readRoomArray(reader);				
			} else if (name.equals("weeksList") && reader.peek() != JsonToken.NULL) {
				// it's OK to expect that dayNumber isn't -1 due to the sequential nature of this code
				weeks = readWeeksArray(reader, dayNumber, start, end);
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new Timetable(dayNumber, start, end, room, weeks);
	}
	
	private static List<Attendance> readWeeksArray(JsonReader reader, int dayNumber, String start, String end) throws IOException {
		List<Attendance> weeks = new ArrayList<Attendance>();
        
		reader.beginArray();
		while (reader.hasNext()) {
			int weekNumber = reader.nextInt();
			
	        GregorianCalendar calStart = parseTime(start, dayNumber, weekNumber);
	        GregorianCalendar calEnd = parseTime(end, dayNumber, weekNumber);
	        
	        // gets the duration in minutes of the activity
	        long difference  = calEnd.getTimeInMillis() - calStart.getTimeInMillis();
	        int duration = (int) TimeUnit.MILLISECONDS.toMinutes(difference);

			weeks.add(new Attendance(calStart, calEnd, duration, false, false));
		}
		reader.endArray();
		
		return weeks;
	}
	
	private static GregorianCalendar parseTime(String time, int dayNumber, int weekNumber) {
        // get correct dates from current year, a given week number, and a day number
		// TODO: we assume that the device is using correct time settings
        GregorianCalendar cal = new GregorianCalendar();
        cal.set(Calendar.DAY_OF_WEEK, dayNumber+2); // NTNU data uses monday = 0, Java uses monday = 2
        cal.set(Calendar.WEEK_OF_YEAR, weekNumber);
        
        int hour = Integer.parseInt(time.substring(0, time.indexOf(":")).toString());
        int minute = Integer.parseInt(time.substring(time.indexOf(":") + 1, time.length()).toString());
        
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.MINUTE, minute);
        cal.set(Calendar.SECOND, 0);
        
        return cal;
	}
	
	private static List<Room> readRoomArray(JsonReader reader) throws IOException {
		List<Room> rooms = new ArrayList<Room>();

		reader.beginArray();
		while (reader.hasNext()) {
			rooms.add(readRoom(reader));
		}
		reader.endArray();
		
		return rooms;
	}
	
	private static Room readRoom(JsonReader reader) throws IOException {
		String name = null;
		String lydiaCode = null;

		reader.beginObject();
		while (reader.hasNext()) {
			String n = reader.nextName();
			if (n.equals("location")) { // name
				name = reader.nextString();
			} else if (n.equals("lydiaCode")) {
				lydiaCode = reader.nextString();
			} else {
				reader.skipValue();
			}
		}
		reader.endObject();

		return new Room(name, lydiaCode);
		//return RoomParser.parseRoom(lydiaCode);
	}

}