package no.skarasoftwaredevelopment.schooltracker.pojo;

import java.io.Serializable;

public class Room implements Serializable {

	public static final String TAG = "Room";
	public static final int BOUNDARY_RADIUS_SMALL_THRESHOLD = 10;
	public static final int BOUNDARY_RADIUS_MEDIUM_THRESHOLD = 20;
	public static final int BOUNDARY_RADIUS_SMALL_ROOM_FACTOR = 4;
	public static final int BOUNDARY_RADIUS_MEDIUM_ROOM_FACTOR = 3;
	public static final int BOUNDARY_RADIUS_LARGE_ROOM_FACTOR = 2;
	
	private static final long serialVersionUID = -2126425909599826123L;
	
	private int id;
	private String nr;
	private String name; // location
	private String type;
	private int buildingId;
	private int floorId;
	private String lydiaCode;
	private float floor;
	private Coord geoCenter;
	private int boundaryRadius;

	/**
	 * This is the preferred constructor to use when
	 * adding rooms to the database.
	 * 
	 * @param id
	 * @param nr
	 * @param name
	 * @param type
	 * @param buildingId
	 * @param floorId
	 * @param lydiaCode
	 * @param geoBoundary
	 */
	public Room(int id, String nr, String name, String type, int buildingId, int floorId, String lydiaCode, Coord[] geoBoundary, float floor) {
		this.id = id;
		this.nr = nr;
		this.name = name;
		this.type = type;
		this.buildingId = buildingId;
		this.floorId = floorId;
		this.lydiaCode = lydiaCode;
		setGeoBoundary(geoBoundary);
		this.floor = floor;
	}
	
	/**
	 * This is the preferred constructor to use when
	 * retrieving rooms from the database.
	 * 
	 * @param id
	 * @param nr
	 * @param name
	 * @param type
	 * @param buildingId
	 * @param floorId
	 * @param lydiaCode
	 * @param geoBoundary
	 */
	public Room(int id, String nr, String name, String type, int buildingId, int floorId, String lydiaCode, Coord geoCenter, int floor, int boundaryRadius) {
		this.id = id;
		this.nr = nr;
		this.name = name;
		this.type = type;
		this.buildingId = buildingId;
		this.floorId = floorId;
		this.lydiaCode = lydiaCode;
		this.geoCenter = geoCenter;
		this.floor = floor;
		this.boundaryRadius = boundaryRadius;
	}
	
	/**
	 * This is the preferred constructor to use
	 * when parsing rooms in {@code RoomParser}.
	 * 
	 * @param name
	 * @param lydiaCode
	 */
	public Room(String name, String lydiaCode) {
		this.name = name;
		this.lydiaCode = lydiaCode;		
	}
	
	/**
	 * This constructor was used before we
	 * had any coordinates in the database.
	 * 
	 * Also used by {@code RoomParser} if
	 * we're getting data about rooms from NTNU.
	 * 
	 * @param id
	 * @param nr
	 * @param name
	 * @param type
	 * @param buildingId
	 * @param floorId
	 * @param lydiaCode
	 */
	public Room(int id, String nr, String name, String type, int buildingId, int floorId, String lydiaCode) {
		this.id = id;
		this.nr = nr;
		this.name = name;
		this.type = type;
		this.buildingId = buildingId;
		this.floorId = floorId;
		this.lydiaCode = lydiaCode;
	}

	public int getId() {
		return id;
	}

	public String getNr() {
		return nr;
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public int getBuildingId() {
		return buildingId;
	}

	public int getFloorId() {
		return floorId;
	}

	public String getLydiaCode() {
		return lydiaCode;
	}
	
	private void setGeoBoundary(Coord[] geoBoundary) {
		calculateGeoCenter(geoBoundary);
		
		// calculate the farthest away from each other
		// the GPS coordinates are
		int tempRadius = 0;
		for (Coord c1 : geoBoundary) {
			for (Coord c2 : geoBoundary) {
				if (!c1.equals(c2)) {
					tempRadius = c1.distanceInM(c2);
					if (tempRadius > boundaryRadius) {
						boundaryRadius = tempRadius;
					}
				}
			}
		}
		
		// boundary radius is currently calculated to be either
		// small, medium, or large depending on the size of the
		// initial radius. a smaller initial radius would use a
		// larger factor while a larger initial radius would 
		// use a smaller factor. this gives us the  longest
		// span between the given GPS coordinates to ensure we
		// have some tolerance to work with
		if (boundaryRadius <= BOUNDARY_RADIUS_SMALL_THRESHOLD) {
			boundaryRadius *= BOUNDARY_RADIUS_SMALL_ROOM_FACTOR;
		} else if (boundaryRadius <= BOUNDARY_RADIUS_MEDIUM_THRESHOLD) {
			boundaryRadius *= BOUNDARY_RADIUS_MEDIUM_ROOM_FACTOR;
		} else {
			boundaryRadius *= BOUNDARY_RADIUS_LARGE_ROOM_FACTOR;
		}
	}
	
	/**
	 * http://www.geomidpoint.com/calculation.html
	 * 
	 * Method A in the URL above describes the following
	 * algorithm for calculating the geographic midpoint
	 * given an arbitrary amount of GPS coordinates.
	 * 
	 * Not using weights in this case, instead each of
	 * the coordinates is weighted 1.
	 */
	private void calculateGeoCenter(Coord[] geoBoundary) {
		double x = 0;
		double y = 0;
		double z = 0;
		
		for (int i = 0; i < geoBoundary.length; i++) {
			Coord c = geoBoundary[i];
			
			// convert lat/lon from degrees to radians
			double lat = c.getLatitude() * (Math.PI / 180);
			double lon = c.getLongitude() * (Math.PI / 180);
			
			// convert lat/lon to Cartesian coordinates
			x += Math.cos(lat) * Math.cos(lon);
			y += Math.cos(lat) * Math.sin(lon);
			z += Math.sin(lat);
		}
		
		// computes the average
		x /= geoBoundary.length;
		y /= geoBoundary.length;
		z /= geoBoundary.length;
		
		// convert average to lon/lat
		double lon = Math.atan2(y, x);
		double hyp = Math.sqrt(x * x + y * y);
		double lat = Math.atan2(z, hyp);
		
		// converting back to degrees from radians
		lat *= 180 / Math.PI;
		lon *= 180 / Math.PI;
		
		geoCenter = new Coord(lon, lat);
	}
	
	public Coord getGeoCenter() {
		return geoCenter;
	}
	
	public float getFloor() {
		return floor;
	}
	
	public int getBoundaryRadius() {
		return boundaryRadius;
	}
	
	/**
	 * Checks if the coord parameter is closer than the calculated 
	 * boundary radius to this coord, meaning that the given coord
	 * counts as inside the given room.
	 * @param coord
	 * @return
	 */
    public boolean isPositionInside(Coord coord) { 
    	// TODO: should we include  && coord.getZ() == floor in the if? (same floor checking)    	
    	if (geoCenter.distanceInM(coord) <= boundaryRadius) {
			return true;
    	} else return false;
	}

	@Override
	public String toString() {
		return "Room [id=" + id + ", nr=" + nr + ", name=" + name + ", type="
				+ type + ", buildingId=" + buildingId + ", floorId=" + floorId
				+ ", lydiaCode=" + lydiaCode + ", geoCenter=" + geoCenter
				+ "]";
	}

}