package no.skarasoftwaredevelopment.schooltracker.pojo;

import java.io.Serializable;
import java.util.Calendar;

import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import android.util.Log;

public class TaskData implements Serializable {
	
	public static final String TAG = "TaskData";

    private static final long serialVersionUID = 951105969340827617L;

	private int taskId;
	private int studentId;
	private String studentUUID;
	private Attendance attendance;
	private int counter;
	private int numberOfRuns;
	private int threshold;
	private int positionsInside;
	private boolean canceled;
	
	public TaskData(Student student, Attendance attendance) {
		this.taskId = attendance.getId();
		this.studentId = student.getId();
		this.studentUUID = student.getUUID();
		this.attendance = attendance;
		
		Log.e(TAG, "initializing task (" + taskId + ")..");
		
		counter = 0;
		canceled = false;
	
		// calculate number of runs based on the duration and interval we need to add one additional run to
		// the total otherwise each task would stop ConstantHandler.POSITION_INTERVAL_REGULAR_MIN minutes before
		// the activity actually ends
		numberOfRuns = (int) Math.ceil((double) attendance.getDuration() / (double) ConstantHandler.POSITION_INTERVAL_REGULAR_MIN) + 1;
		Log.e(TAG, "taskId (" + taskId + "): number of runs = " + numberOfRuns);
		
		// we cast the result of the division to an int to cut off the decimals (effectively rounding down)
		// there is no need to take into account the extra ConstantHandler.POSITION_INTERVAL_REGULAR_MIN minutes
		// here as an activity never will end this close to it's next pause anyhow
		int numberOfBreaks = (int) (attendance.getDuration() / ConstantHandler.LECTURE_BLOCK_INCLUDING_BREAK);
		int breaksInRuns = (numberOfBreaks * ConstantHandler.BREAK_LENGTH) / ConstantHandler.POSITION_INTERVAL_REGULAR_MIN;

		// we need to account for the breaks between each lecture block since we don't want to punish the
		// student for leaving the designated room(s) during the activity's life span. we only require the
		// student to attend 75% of the lecture, which is also included in this calculation.
		int runsNoBreaks = numberOfRuns - breaksInRuns;

		// calculate the threshold for the lowest amount of positions required for the student to be
		// considered as attended by the app. this is only applicable to lecture type activities.
		threshold = (int) Math.ceil((double) runsNoBreaks * ConstantHandler.ATTENDANCE_THRESHOLD_FACTOR);
		Log.e(TAG, "taskId (" + taskId + "): threshold = " + threshold);
		
		// due to the nature of alarms and thread handling, we need to account for slight
		// variances in the run times of each task, meaning that the last task need to be
		// able to run a few seconds/minutes after the activity ended. this will not cause
		// any issues with the number of times the task is run due to the counter we have.
		attendance.getEndDate().add(Calendar.MINUTE, ConstantHandler.POSITION_INTERVAL_REGULAR_MIN);
	}
	
	public int getTaskId() {
		return taskId;
	}
	
	public int getStudentId() {
		return studentId;
	}
	
	public String getStudentUUID() {
		return studentUUID;
	}
	
	public Attendance getAttendance() {
		return attendance;
	}
	
	public int getCounter() {
		return counter;
	}
	
	public void incrementCounter() {
		counter++;
	}
	
	public int getNumberOfRuns() {
		return numberOfRuns;
	}
	
	public int getThreshold() {
		return threshold;
	}
	
	public int getPositionsInside() {
		return positionsInside;
	}
	
	public void setPositionsInside(int inside) {
		this.positionsInside = inside; 
	}
	
	public boolean isCanceled() {
		return canceled; 
	}

	public void setCanceled(boolean canceled) {
		this.canceled = canceled;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + taskId;
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof TaskData))
			return false;
		TaskData other = (TaskData) obj;
		if (taskId != other.taskId)
			return false;
		return true;
	}
	
}