package no.skarasoftwaredevelopment.schooltracker.services;

import java.util.ArrayList;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.pojo.TaskData;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiManager.WifiLock;
import android.os.Binder;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.GoogleApiClient.ConnectionCallbacks;
import com.google.android.gms.common.api.GoogleApiClient.OnConnectionFailedListener;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

public class PositionService extends Service implements ConnectionCallbacks, OnConnectionFailedListener, LocationListener {

	public static final String TAG = "PositionService";
	public static final String EXTRA_ATTENDANCE = "no.skarasoftwaredevelopment.schooltracker.services.ATTENDANCE";
	public static final String EXTRA_STUDENT = "no.skarasoftwaredevelopment.schooltracker.services.STUDENT";
	public static final String EXTRA_TASK_DATA = "no.skarasoftwaredevelopment.schooltracker.services.TASK_DATA";
	
	public static boolean isRunning = false;
	public static Location lastKnown;
	public static boolean connected;
	
	private final IBinder BINDER = new LocalBinder();
	
	private static PositionService activeService;

	private List<TaskData> tasks = new ArrayList<TaskData>();
	private GoogleApiClient googleApiClient;
	private LocationRequest locationRequest;
	private WifiLock wifiLock;
	
	public PositionService() {
		activeService = this;
	}
	
	@Override
    public int onStartCommand(Intent intent, int flags, int startId) {
		// signify to the client that the service is indeed started
		if (!isRunning || googleApiClient == null) {
			isRunning = true;
			
			// connect to google play services and create a
			// location request used by the LocationServices
			buildGoogleApiClient();
			createLocationRequest();
			
			// make sure we don't lose connection to wifi when
			// the phone is locked by acquiring a wifi lock
			WifiManager wifiManager = (WifiManager) getSystemService(Context.WIFI_SERVICE);
			wifiLock = wifiManager.createWifiLock(WifiManager.WIFI_MODE_FULL, TAG);
			wifiLock.acquire();
		}
		
		if (intent != null) {
			Log.e(TAG, "service received task..");
			
			Attendance attendance = (Attendance) intent.getSerializableExtra(EXTRA_ATTENDANCE);
			if (attendance != null) {
				int taskId = attendance.getId();
				
				if (!taskExists(taskId)) {				
					Student student = (Student) intent.getSerializableExtra(EXTRA_STUDENT);
					
					TaskData taskData = new TaskData(student, attendance);
					addTask(taskData);
					
					Intent i = new Intent(this, PositionTaskService.class);
					i.putExtra(EXTRA_TASK_DATA, taskData);
					
			        startService(i);
				} else {
					Log.e(TAG, "task (" + taskId + ") is already added");
				}
			}
		} else {
			Log.e(TAG, "no intent, the service was restarted by the system..");
	    	SchedulingService.scheduleAlarms(this);
		}
		
        return Service.START_STICKY; // makes sure the service is restarted if killed by the system
    }
	
	public synchronized static PositionService getActiveService() {
		return activeService;
	}
	
	/**
	 * Gives the client a way to interact with the service.
	 * This is the preferred way of communicating with the service.
	 * 
	 * @author Tommy
	 *
	 */
	public class LocalBinder extends Binder {
		public PositionService getService() {
            return PositionService.this;
        }
    }
	
    /**
     * This method checks if the task has already been
     * added by checking if the list includes the new
     * taskId. The difference between this and task.contains()
     * is that we don't have to create the PositionTask
     * object first, which automatically starts a location
     * listener even though the task is already added.
     * This prevents us from having multiple unnecessary
     * listeners as a result of trying to add existing tasks.
     * 
     * @param taskId
     * @return
     */
    private boolean taskExists(int taskId) {
    	for (TaskData task : tasks) {
    		if (task.getTaskId() == taskId) {
    			return true;
    		}
    	}
    	
    	return false;
    }
    
    /**
     * Add a new task only if it doesn't already exist.
     * 
     * @param position
     * @return
     */
    private void addTask(TaskData task) {
    	if (!tasks.contains(task)) {
    		tasks.add(task);
			Log.e(TAG, "adding task (" + task.getTaskId() + ")");
    	}
    }
    
    /**
     * Cancels and removes all tasks attached to a course.
     * given the course code. This does not stop currently
     * running tasks as the reference to the given tasks
     * is lost when the TaskData object is sent to the
     * IntentService as an extra. Not a huge problem as the
     * tasks won't be recurring, only run until they're done.
     * 
     * @param courseCode
     */
    public void cancelAllTasks(String courseCode) {
    	if (courseCode != null) {
	    	for (int i = tasks.size() - 1; i >= 0; i--) {
	    		TaskData task = tasks.get(i);	
	    		if (task.getAttendance().getCourseCode().equalsIgnoreCase(courseCode)) {
	    			cancelTask(task);
	    		}
	    	}
    	}
    }
    
    /**
     * Cancels and removes the given task. This does not
     * stop currently running task as the reference to
     * the given task is lost when the TaskData object
     * is sent to the IntentService as an extra. Not a
     * huge problem as the task won't be recurring, only
     * run until it is done.
     * 
     * @param position
     */
    public void cancelTask(TaskData task) {    	
    	if (task != null) {
    		// does nothing as of now since this task isn't referencing the
    		// same task as the one currently being used by PositionTaskService
    		//task.setCanceled(true);
    		
    		tasks.remove(task);
    		
    		Log.e(TAG, "removing task (" + task.getTaskId() + ")..");
    		Log.e(TAG, getNumberOfTasksRunning() + " tasks remaining");
    	}
    	
    	stopService();
    }
    
    /**
     * Stops the service if the last task in the list
     * was cancelled and removed.
     */
    private void stopService() {
    	if (getNumberOfTasksRunning() == 0) {    		
    		stopSelf();
    	}
    }
	
	/**
	 * Gets the number of tasks currently running.
	 * 
	 * @return size of the tasks list
	 */
	public int getNumberOfTasksRunning() {
		return tasks.size();
	}
	
	@Override
	public void onDestroy() {
		Log.e(TAG, "no more tasks running, stopping service..");
		
		isRunning = false;
		
		// disconnect gps
		if (googleApiClient != null && googleApiClient.isConnected()) {
			LocationServices.FusedLocationApi.removeLocationUpdates(googleApiClient, this);
			googleApiClient.disconnect();
		}
		
		// cleanup the wifi lock
		wifiLock.release();
		
		super.onDestroy();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
	    return BINDER;
	}
	
	public static synchronized Location getLastKnown() {
		return lastKnown;
	}
	
	public static synchronized boolean isConnected() {
		return connected;
	}
	
	private void buildGoogleApiClient() {
		googleApiClient = new GoogleApiClient.Builder(this)
	        .addConnectionCallbacks(this)
	        .addOnConnectionFailedListener(this)
	        .addApi(LocationServices.API)
	        .build();

		googleApiClient.connect();
	}
	
	private void createLocationRequest() {
		// creates a request that decides interval/priority
		// of when location updates should take place
		locationRequest = new LocationRequest();
		locationRequest.setInterval(ConstantHandler.POSITION_INTERVAL_REGULAR_MS);
		locationRequest.setFastestInterval(ConstantHandler.POSITION_INTERVAL_FASTEST_MS);
		
		// not sure if this one is accurate enough, according to Google
		// this priority has an accuracy of approximately 100 meters
		// (https://developer.android.com/training/location/receive-location-updates.html)
		// which in most cases isn't good enough for a classroom
		//locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY);
		
		// testing of the above method has shown discrepancies in the results
		// which led us to use this method instead
		locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
	}
	
	@Override
	public void onConnected(Bundle bundle) {
		if (googleApiClient != null && locationRequest != null) {
		    LocationServices.FusedLocationApi.requestLocationUpdates(googleApiClient, locationRequest, this);
		    
		    // in case the device has recently requested a location nearby
		    // then we try to use this before the LocationListener starts
		    // getting positions
		    lastKnown = LocationServices.FusedLocationApi.getLastLocation(googleApiClient);
		}
	}
	
	@Override
	public void onLocationChanged(Location location) {
		Log.e(TAG, "new position..");
		
		//Log.e(TAG, "accuracy: " + location.getAccuracy());
		//Calendar time = Calendar.getInstance();
		//time.set(Calendar.MILLISECOND, (int) location.getTime());
		//Log.e(TAG, "timestamp: " + FormatHandler.dateFormat.format(time.getTime()));
		
		lastKnown = location;
		connected = true;
	}
	
	@Override
	public void onConnectionFailed(ConnectionResult result) {
		Log.e(TAG, "connection failed..");
		connected = false;
	}
	
	@Override
	public void onConnectionSuspended(int error) {
		Log.e(TAG, "connection suspended..");
		connected = false;
	}

}