package no.skarasoftwaredevelopment.schooltracker.services;

import java.util.Calendar;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ConstantHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.LootHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.NotificationHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.ReportPositionHandler;
import no.skarasoftwaredevelopment.schooltracker.parsers.PositionParser;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData;
import no.skarasoftwaredevelopment.schooltracker.pojo.PositionData.PositionProvider;
import no.skarasoftwaredevelopment.schooltracker.pojo.Room;
import no.skarasoftwaredevelopment.schooltracker.pojo.StudentPosition;
import no.skarasoftwaredevelopment.schooltracker.pojo.TaskData;
import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;
import android.os.SystemClock;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;

public class PositionTaskService extends IntentService {

    public static final String TAG = "PositionTaskService";
    public static final String ACTION_BROADCAST = "no.skarasoftwaredevelopment.schooltracker.services.BROADCAST";
    
    public PositionTaskService() {
        super(TAG);
    }

	private TaskData taskData;
	private Attendance attendance;
	private DbHelper db;

    @Override
    protected void onHandleIntent(Intent intent) {
    	taskData = (TaskData) intent.getSerializableExtra(PositionService.EXTRA_TASK_DATA);
    	attendance = taskData.getAttendance();	
    	db = new DbHelper(this);
    	
    	if (taskStillRunning()) {
    		long executionTime = System.currentTimeMillis();
    		getPosition();
    		executionTime = System.currentTimeMillis() - executionTime;
    		
    		// if this wasn't the last execution of the task, we schedule it again
    		if (taskStillRunning()) {
    			scheduleNextTask(executionTime, intent);
    		} else {
    			finishTask();
    		}
    	} else {
    		// there are rare cases where we end up here, like when the task was
    		// scheduled to run but the time it took to get here made it not
    		// eligible to be run anymore.
    		finishTask();
    	}
    	
    	// broadcast the result from running the task (stored in attendance)
    	Intent i = new Intent(ACTION_BROADCAST).putExtra(PositionService.EXTRA_ATTENDANCE, attendance);
		LocalBroadcastManager.getInstance(this).sendBroadcast(i);
    }
    
	private void scheduleNextTask(long previousExecutionTime, Intent i) {		
		// we make sure to subtract the time it took the last task to complete
		// from the interval, this way we limit the time each increment of the
		// task will add to the next execution time (now + msToNextPos)
		Calendar now = Calendar.getInstance();
		int msToNextPos = (int) (ConstantHandler.POSITION_INTERVAL_REGULAR_MS - previousExecutionTime);
		now.add(Calendar.MILLISECOND, msToNextPos);
		
		PendingIntent alarmIntent = PendingIntent.getService(this, attendance.getId(), i, PendingIntent.FLAG_UPDATE_CURRENT);
		AlarmManager alarmManager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
		SchedulingService.setAlarm(alarmManager, now.getTimeInMillis(), alarmIntent);
	}

	private boolean taskStillRunning() {
		Calendar now = Calendar.getInstance();
		return taskData.getCounter() < taskData.getNumberOfRuns() && !taskData.isCanceled() && now.after(attendance.getStartDate()) && now.before(attendance.getEndDate());
	}

    private void getPosition() {		
		Log.e(TAG, "taskId (" + taskData.getTaskId() + "): running task.. (" + (taskData.getCounter() + 1) + "/" + taskData.getNumberOfRuns() + ")");
		taskData.incrementCounter();
		
		checkNetworkCapabilities();
		
		// we try to get a position from MazeMap first
		PositionData pos = PositionParser.parsePosition();
		
		if (pos != null) {
			// we succeeded in getting a position from MazeMap
			attendance.setInside(isInside(pos));
			Log.e(TAG, "taskId (" + taskData.getTaskId() + "): MazeMap location found..");
		} else {
			// we couldn't connect to MazeMap or we got an error
			attendance.setInside(false);
			Log.e(TAG, "taskId (" + taskData.getTaskId() + "): MazeMap unavailable, use fallback method..");
		}
		
		// if MazeMap/MSE fails us, we fall back on the LocationManager
		if (!attendance.isInside()) {
			pos = getGooglePosition();
			
			// no reason to check if the position is inside any of the rooms
			// if we know for a fact that the position is a dummy position
			if (pos.getProvider() != PositionProvider.DUMMY) {
				attendance.setInside(isInside(pos));
				Log.e(TAG, "taskId (" + taskData.getTaskId() + "): Google location found..");
			}
		}
		
		Log.e(TAG, "getting position for " + attendance.getCourseCode() + " (taskId: " + taskData.getTaskId() + ").." + attendance.isInside());

		// helps encapsulate all the data we wish to pass on to the
		// two methods below
		StudentPosition studentPos = new StudentPosition(
				taskData.getStudentId(), attendance.getId(),
				String.valueOf(pos.getCoord().getLongitude()),
				String.valueOf(pos.getCoord().getLatitude()),
				pos.getZ(), attendance.isInside(), pos.getProvider());			

		// saves the current position (in case we need it later)
		// as well as the attendance the position is connected to
		// and if the position was inside the room(s) or not
		db.addStudentPosition(studentPos);
		ReportPositionHandler.report(taskData.getStudentUUID(), attendance.getCourseCode(), studentPos);
    }
    
    /**
     * Checks whether or not the given position is inside
     * any of the rooms linked to this task's attendance.
     * 
     * @param pos
     * @return
     */
    private boolean isInside(PositionData pos) {
    	boolean inside = false;
    	
    	List<Room> rooms = attendance.getRooms();
		for (Room room : rooms) {
			// we only need to know that the student
			// is inside one of the possible rooms
			inside = room.isPositionInside(pos.getCoord());
			
			if (inside) {
				break;
			}
		}
		
		return inside;
    }
    
    private PositionData getGooglePosition() {
    	PositionData pos = PositionData.getDummy();

		if (PositionService.isConnected()) {
			Location lastKnown = PositionService.getLastKnown();
			
			if (lastKnown != null) {
				// we accurately measure how much time has passed between now and when the location was recorded
				long differenceInMs = (SystemClock.elapsedRealtimeNanos() - lastKnown.getElapsedRealtimeNanos()) / 1000000;
				
				if (differenceInMs <= ConstantHandler.POSITION_MAX_AGE_MS) {
					// create PositionData from Location
					pos = new PositionData(lastKnown.getLongitude(), lastKnown.getLatitude());
					Log.e(TAG, "taskId (" + taskData.getTaskId() + "): GPS location found..");
				} else {
					// the last known GPS location is too old, and so we can't trust it
					Log.e(TAG, "taskId (" + taskData.getTaskId() + "): stale data..");
				}
			} else {
				// there isn't a location available (turned off the gps?) so we need
				// to request a dummy position to avoid null pointers below
				Log.e(TAG, "taskId (" + taskData.getTaskId() + "): no last known GPS location..");
			}
		} else {
			// we don't have any positional data whatsoever so we request a dummy
			// position to avoid null pointers below
			Log.e(TAG, "taskId (" + taskData.getTaskId() + "): no positional data available..");
		}
		
		return pos;
    }
    
    private void finishTask() {
		// when we exit the while loop that means the work is done and that
		// the attendance has passed
		attendance.setPassed(true);
		
		// gets the number of positions that was inside the required room(s).
		// doing it this way ensures us that the data used here reflects the
		// data in the database. we avoid problems where the PositionService,
		// the app, or the device was restarted during the task or the course
		// was added during the task.
		taskData.setPositionsInside(db.getStudentPositionsInside(1, attendance.getId())); // TODO: get studentId dynamically
		
		// we require students to attend a set portion of any given lecture
		// for it to be counted as attended, but for exercises they only need
		// one position to be inside for it to counted as attended
    	attendance.checkAttendance(taskData);

		// saves the result/status about the attendance based on positions collected during the activity
		db.updateAttendance(1, attendance.getId(), attendance.hasAttended(), attendance.hasPassed()); // TODO: should get studentId dynamically
		
		// forces the GUI to refresh the effort data for the given course
		db.setStudentCourseEffortUpToDate(1, attendance.getCourseCode(), false); // TODO: should get studentId dynamically
		
		// the student has a chance to generate loot if he has attended the activity
		LootHandler.checkEligibility(attendance.hasAttended(), db, this);
			
		// check for and add new achievements, and then notify the user with a notification
		AchievementHandler.checkEligibility(db, this);
		
		Log.e(TAG, "status (" + taskData.getTaskId() + "): " + taskData.getPositionsInside() + " / " + taskData.getThreshold() + " (" + taskData.getNumberOfRuns() + ")");
		Log.e(TAG, "stopping task (" + taskData.getTaskId() + ")..");
		
		// removes network notification when finishing this task
		NotificationHandler.removeNotification(this, NotificationHandler.NETWORK);
		
		// a cleaner way of doing this would be to use a Binder but this
		// IntentService might finish before the connection is made 
		PositionService.getActiveService().cancelTask(taskData);
    }
    
    /**
     * Notifies the student if GPS, Wi-Fi, or the mobile network isn't enabled.
     */
    private void checkNetworkCapabilities() {
	    // GPS
    	LocationManager locManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
	    boolean gpsEnabled = locManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
	    
	    // Wi-Fi
	    WifiManager wifiManager = (WifiManager) getSystemService(Context.WIFI_SERVICE);
	    boolean wifiEnabled = wifiManager.isWifiEnabled();
    	
	    // mobile network
    	ConnectivityManager conManager = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
	    NetworkInfo networkInfo = conManager.getActiveNetworkInfo();
	    boolean networkEnabled = false;
	    
	    if (networkInfo != null && networkInfo.isConnected()) {
	    	networkEnabled = true;
	    }
	    
	    // either display or remove notification
	    if (!gpsEnabled || !wifiEnabled || !networkEnabled) {
	    	NotificationHandler.notifyNetwork(this);
	    } else {
	    	NotificationHandler.removeNotification(this, NotificationHandler.NETWORK);
	    }
    }

}