package no.skarasoftwaredevelopment.schooltracker.services;

import java.util.Calendar;
import java.util.List;

import no.skarasoftwaredevelopment.schooltracker.database.DbHelper;
import no.skarasoftwaredevelopment.schooltracker.handlers.AchievementHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.FormatHandler;
import no.skarasoftwaredevelopment.schooltracker.handlers.LootHandler;
import no.skarasoftwaredevelopment.schooltracker.pojo.Attendance;
import no.skarasoftwaredevelopment.schooltracker.pojo.Student;
import no.skarasoftwaredevelopment.schooltracker.pojo.TaskData;
import no.skarasoftwaredevelopment.schooltracker.receivers.AlarmReceiver;
import android.annotation.TargetApi;
import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;

/**
 * This class handles the automatic and manual scheduling of alarms.
 * 
 * The automatic scheduling happens each day
 * at 03:00. {@code SchedulingService} is run by AlarmReceiver
 * (a {@code WakefulBroadcastReceiver} which maintains a partial
 * wake lock for this service. When the service is done (after
 * running onHandleIntent()) it releases the wake lock again
 * by calling {@code completeWakefulIntent()}.
 * 
 * The manual scheduling happens whenever the student adds a
 * new course. {@code AddCourseFragment} runs {@code scheduleAlarms()}.
 * 
 * @author Tommy
 *
 */
public class SchedulingService extends IntentService {

    public static final String TAG = "SchedulingService";
	
    public SchedulingService() {
        super(TAG);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
    	Log.e(TAG, "checking for alarms to schedule..");

    	scheduleAlarms(this);
    	
        // releases the wake lock provided by the BroadcastReceiver
        AlarmReceiver.completeWakefulIntent(intent);
    }

    public static void scheduleAlarms(Context c) {
    	AlarmManager alarmManager = (AlarmManager) c.getSystemService(Context.ALARM_SERVICE);
    	PendingIntent alarmIntent;
    	
    	DbHelper db = new DbHelper(c);
    	Student student = db.getStudent(1); // TODO: get studentId dynamically
    	
    	// today's date
    	Calendar now = Calendar.getInstance();
    	int dayNumber = now.get(Calendar.DAY_OF_WEEK) - 2; // subtract 2 to make it compatible with NTNU's formatting
    	int weekNumber = now.get(Calendar.WEEK_OF_YEAR);
    	
    	// get today's attendances and schedule alarms
		// if there are any activities happening today
    	List<Attendance> attendances = db.getStudentAttendances(1, dayNumber, weekNumber); // TODO: get studentId dynamically
    	
    	// register alarms to start PositionService at the correct times
    	for (Attendance attendance : attendances) {
    		// we don't want to schedule more alarms/tasks for activities already attended
    		if (!attendance.hasAttended() && !attendance.hasPassed()) {
	            Intent i = new Intent(c, PositionService.class);
	            i.putExtra(PositionService.EXTRA_ATTENDANCE, attendance);
	            i.putExtra(PositionService.EXTRA_STUDENT, student);

	            // we use a request code (the attendanceId) to separate
	            // this pending intent from other pending intents 
	            alarmIntent = PendingIntent.getService(c, attendance.getId(), i, PendingIntent.FLAG_UPDATE_CURRENT);
	            
	            if (attendance.getStartDate().after(now) && attendance.getEndDate().after(now)) {
	            	// the alarm is later in the day
	            	setAlarm(alarmManager, attendance.getStartDate().getTimeInMillis(), alarmIntent);
		            Log.e(TAG, "adding alarm: " + attendance.getId() + " (duration: " + attendance.getDuration() + ")");
	            } else if (attendance.getStartDate().before(now) && attendance.getEndDate().after(now)) {
	            	// TODO: in this case, we should probably check how many positions have
	            	// previously been recorded to ensure that the student can't record
	            	// additional positions beyond what's expected (numberOfRuns) by
	            	// changing the time settings on the device.
	            	
	            	// the alarm should be fired right away
	            	setAlarm(alarmManager, 0, alarmIntent);
		            Log.e(TAG, "adding alarm (NOW!): " + attendance.getId() + " (duration: " + attendance.getDuration() + ")");
	            } else {
	            	// if the attendance isn't later today or happening right now
	            	// then that means it has already passed today
	            	attendance.setPassed(true);
	            	
	            	// initializing a TaskData object calculates threshold for a
	            	// given attendance, which is what we need to do here
	            	TaskData task = new TaskData(student, attendance);
	        		
	        		// gets the number of positions that was inside the required room(s)
	            	task.setPositionsInside(db.getStudentPositionsInside(1, attendance.getId())); // TODO: get studentId dynamically
				
	        		// we require students to attend a set portion of any given lecture
	        		// for it to be counted as attended, but for exercises they only need
	        		// one position to be inside for it to counted as attended
	            	attendance.checkAttendance(task);
					
					// saves the result/status about the attendance based on positions collected during the activity
					db.updateAttendance(1, attendance.getId(), attendance.hasAttended(), attendance.hasPassed()); // TODO: should get studentId dynamically
					
					// forces the GUI to refresh the effort data for the given course
					db.setStudentCourseEffortUpToDate(1, attendance.getCourseCode(), false); // TODO: should get studentId dynamically
					
					// the student has a chance to generate loot if he has attended the activity
					LootHandler.checkEligibility(attendance.hasAttended(), db, c);

					// check for and add new achievements, and then notify the user with a notification
					AchievementHandler.checkEligibility(db, c);
					
					Log.e(TAG, "today's class has already passed (" + FormatHandler.dateFormat.format(attendance.getEndDate().getTime()) + "), don't schedule alarm");
	            }
    		} else {
    			Log.e(TAG, "this activity has already been attended, no need to schedule an alarm");
    		}
    	}
    }
    
    /**
     * Depending on what version of Android the device is running
     * the method will use the following ways to set an alarm.
     * 
     * KITKAT or higher:
     * Will use {@code setWindow()} because this method was made
     * available in this version of Android and is recommended by Google
     * for cases where a high degree of precision within the window
     * is required.
     * 
     * Lower than KITKAT:
     * Will use {@code set()} because this method is supported all
     * the way back to version 1 of Android.
     * 
     * RTC_WAKEUP is used in both alarms and fires according to
     * the wall time on the device ({@code System.currentTimeInMillis()}).
     * We also assume that all students use the same {@code TimeZone}
     * as this app is targeting NTNU.
     * 
     * If we were to support different time zones/locales we would use
     * {@code AlarmManager.ELAPSED_REALTIME_WAKEUP} instead and
     * calculate the alarm time based on the amount of time that
     * has passed since the device was booted (including sleep).
     * 
     * The formulae for when to start an alarm would be:
     * {@code SystemClock.elapsedRealtime() + (startTime - now.getTimeInMillis())}
     * 
     * @param alarmManager
     * @param startTime
     * @param alarmIntent
     */
    @TargetApi(Build.VERSION_CODES.KITKAT)
    public static void setAlarm(AlarmManager alarmManager, long startTime, PendingIntent alarmIntent) {    	
    	if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
	    	alarmManager.setWindow(
	        		AlarmManager.RTC_WAKEUP,
	        		startTime,
	        		1000, // 1 second window to start the alarm
	        		alarmIntent);
    	} else {
            alarmManager.set(
            		AlarmManager.RTC_WAKEUP,
            		startTime,
            		alarmIntent);
    	}
    }

}